#include <GL/openglut.h>
#include <SOIL.h>
#include <vector>
#include <string>

#include "util.cpp"
using namespace std;


class GUI
{
    public:
        int x,y,width,height;
    public:
        bool visible;
        bool selected;
        bool focus;
        virtual void draw()=0;
        virtual void checkClick(int key,int state,int mx,int my)=0;
        virtual void checkKey(int key,int state)=0;
        virtual void setPosition(int X,int Y)=0;
        virtual void setSize(int W,int H)=0;
        GUI(int X,int Y,int W,int H):x(X),y(Y),width(W),height(H){}
};

class TextBox : public GUI
{
    public:
        string str;
        float length;
        int pos;
        int textures[1];
        Font font;
        
        
        TextBox():GUI(0,0,0,0){}
        
        TextBox(int X,int Y,int W,int H,Font F,string STR="",vector<char*>img = vector<char*>(0)):GUI(X,Y,W,H)
        {
            pos=0;
            font=F;
            setText(STR);
            
            visible=true;
            selected=false;
            focus=true;
            
            if(img.size()==0)
            {
                img.push_back("back.jpg");
                img.push_back("back2.jpg");
            }
            
            for(int i=0;i<img.size();i++)
            {
                textures[i]=SOIL_load_OGL_texture
                        	(
                        		img[i],
                        		SOIL_LOAD_AUTO,
                        		SOIL_CREATE_NEW_ID,
                        		//SOIL_FLAG_POWER_OF_TWO | SOIL_FLAG_MIPMAPS | SOIL_FLAG_DDS_LOAD_DIRECT  , SOIL_FLAG_INVERT_Y , | SOIL_FLAG_NTSC_SAFE_RGB
                        		SOIL_FLAG_MIPMAPS  | SOIL_LOAD_RGBA  | SOIL_FLAG_COMPRESS_TO_DXT
                        	);
                if(textures[i]<0)
                {
                    MessageBoxA(NULL,SOIL_last_result(),"ERROR",1);
                }
            }
        }
        
        void setText(string text)
        {
            str=text;
            length=0;
            pos=str.size();
            
            while(length<width-font.size*2 && pos>=0)
            {
                length+=font.xsize[str[pos]];
                pos--;
            }
            pos++;
            
        }
        
        virtual void setSize(int W,int H)
        {
            width=W;
            height=H;
            setText(str);
        }
        virtual void setPosition(int X,int Y)
        {
            x=X;
            y=Y;
        }
        virtual void checkClick(int key,int state,int mx,int my)
        {
            if(key==0)
            {
                if(state==0)                                                //push
                {
                    if(mx>x && mx<x+width && my>y && my<y+height)
                    {
                        selected=true;
                        focus=true;
                    }else
                    {
                        selected=false;
                        focus=false;
                    }
                }
            }
        }
        virtual void checkKey(int key,int state)
        {
            if(selected)
            {    
                if(state==0)
                {
                    if(key==8)
                    {
                        if(str.size()>0)
                            setText(str.substr(0,str.size()-1));
                    }else
                    {
                        setText(str+(char)(key-7));    
                    }
                }
            }        
        }
        virtual void draw()
        {
            if(visible)
            {
                glEnable(GL_TEXTURE_2D);
                        if(focus)
                            glBindTexture(GL_TEXTURE_2D,textures[0]);
                        else
                            glBindTexture(GL_TEXTURE_2D,textures[1]);
                    glBegin(GL_QUADS);
        				glTexCoord2f(0,1);			
        				glVertex2i(x,y+height);		
        				glTexCoord2f(1,1);	
        				glVertex2i(x+width,y+height);
        				glTexCoord2f(1,0);			
        				glVertex2i(x+width,y);		
        				glTexCoord2f(0,0);			
        				glVertex2i(x,y);			
        			glEnd();			
                glDisable(GL_TEXTURE_2D);
                    
                glPushMatrix();
                    glPrint((int)(x+(width-length)/2.0),(int)(y+(height-font.size)/2.0),font,str.substr(pos).c_str());
                glPopMatrix();
            }
        }
};

class MsgBox : public GUI
{
    public:
        Font font;
        string str,real;
        float length;
        int textures[1];
        bool background;
        bool forceResize;
        
        MsgBox():GUI(0,0,0,0){}
        
        MsgBox(int X,int Y,int W,int H,Font &F,string STR="",bool BACK=true,bool FORCE=true,vector<char*>img = vector<char*>(0)):GUI(X,Y,W,H)
        {
            font=F;
            background=BACK;
            forceResize=FORCE;
            setText(STR);            
            
            visible=true;
            selected=false;
            focus=true;
            
            if(img.size()==0)
            {
                img.push_back("back.jpg");
            }
            
            for(int i=0;i<img.size();i++)
            {
                textures[i]=SOIL_load_OGL_texture
                        	(
                        		img[i],
                        		SOIL_LOAD_AUTO,
                        		SOIL_CREATE_NEW_ID,
                        		//SOIL_FLAG_POWER_OF_TWO | SOIL_FLAG_MIPMAPS | SOIL_FLAG_DDS_LOAD_DIRECT  , SOIL_FLAG_INVERT_Y , | SOIL_FLAG_NTSC_SAFE_RGB
                        		SOIL_FLAG_MIPMAPS  | SOIL_LOAD_RGBA  | SOIL_FLAG_COMPRESS_TO_DXT
                        	);
                if(textures[i]<0)
                {
                    MessageBoxA(NULL,SOIL_last_result(),"ERROR",1);
                }
            }
        }
        
        void setText(string text)
        {
            real=text;
            str=text;
            length=font.getLength(str);
            
            if(forceResize)
            {
                if(length>width-font.size*2)
                {
                    width=(int)length+font.size*2;
                }
                
                if(font.size>height)
                {
                    height=font.size*3/2;
                }
            }
            else
            {
                if(length>width-font.size*2)
                {
                    int pos=0;
                    length=0;
                    while(length<width-font.size*2)
                    {
                        length+=((float)font.xsize[str[pos]]*font.scale);
                        pos++;
                    }
                    if(pos>3)
                    {
                        str=str.substr(0,pos-4)+"...";
                    }else
                    {
                        str=str.substr(0,pos-1);
                    }
                }
            }
        }
        
        virtual void setSize(int W,int H)
        {
            width=W;
            height=H;
            setText(real);
        }
        
        virtual void setPosition(int X,int Y)
        {
            x=X;
            y=Y;
        }
        
        virtual void checkClick(int key,int state,int x,int y)
        {
        }
        
        virtual void checkKey(int key,int state)
        {
        }    
        
        virtual void draw()
        {   
            if(visible)
            {
                if(background)
                {
                    glEnable(GL_TEXTURE_2D);
                            glBindTexture(GL_TEXTURE_2D,textures[0]);
                        glBegin(GL_QUADS);
            				glTexCoord2f(0,1);			
            				glVertex2i(x,y+height);		
            				glTexCoord2f(1,1);	
            				glVertex2i(x+width,y+height);
            				glTexCoord2f(1,0);			
            				glVertex2i(x+width,y);		
            				glTexCoord2f(0,0);			
            				glVertex2i(x,y);			
            			glEnd();			
                    glDisable(GL_TEXTURE_2D);
                }
                glPushMatrix();
                    glPrint((int)(x+(width-length)/2.0),(int)(y+(height-font.size)/2.0),font,str.c_str());
                glPopMatrix();
            }
        }
};


class Button : public GUI
{
    public:
        MsgBox text;
        
        int textures[3];
        bool pushing;
        void (*onClick)(void);
        void (*onRelease)(void);
        
        Button():GUI(0,0,0,0){}
        
        Button(int X,int Y,int W,int H,Font f,string STR="",vector<char*>img = vector<char*>(0)):GUI(X,Y,W,H)
        {   
            text=MsgBox(X,Y,W,H,f,STR,false,false);
            
            onClick=NULL;
            onRelease=NULL;
            visible=true;
            selected=false;
            focus=true;
            pushing=false;
            if(img.size()==0)
            {
                img.push_back("defaultB.png");
                img.push_back("defaultBP.png");
                img.push_back("defaultBF.png");
            }
            
            for(int i=0;i<img.size();i++)
            {
                textures[i]=SOIL_load_OGL_texture
                        	(
                        		img[i],
                        		SOIL_LOAD_AUTO,
                        		SOIL_CREATE_NEW_ID,
                        		//SOIL_FLAG_POWER_OF_TWO | SOIL_FLAG_MIPMAPS | SOIL_FLAG_DDS_LOAD_DIRECT  , SOIL_FLAG_INVERT_Y
                        		SOIL_FLAG_MIPMAPS  | SOIL_LOAD_RGBA | SOIL_FLAG_COMPRESS_TO_DXT
                        	);
                if(textures[i]<0)
                {
                    MessageBoxA(NULL,SOIL_last_result(),"ERROR",1);
                }
            }
        }
        
        void setOnClick(void (*f)(void))
        {
            onClick=f;
        }
        void setOnRelease(void (*f)(void))
        {
            onRelease=f;
        }
        
        virtual void setSize(int W,int H)
        {
            width=W;
            height=H;
            text.setSize(W,H);
        }
        
        virtual void setPosition(int X,int Y)
        {
            x=X;
            y=Y;
            text.setPosition(X,Y);
        }
        
        virtual void checkClick(int key,int state,int mx,int my)
        {
            if(key==0)
            {
                if(state==0)                                                //push
                {
                    if(mx>x && mx<x+width && my>y && my<y+height)
                    {
                        pushing=true;
                        selected=true;
                        focus=true;
                        if(onClick!=NULL)
                            onClick();
                    }else
                    {
                        selected=false;
                        focus=false;
                    }
                }else
                {
                    if(pushing==true)
                    {
                        pushing=false;
                        if(mx>x && mx<x+width && my>y && my<y+height)
                        {
                            if(onRelease!=NULL)
                                onRelease();
                        }
                    }
                }
            }
        }
        
        virtual void checkKey(int key,int state)
        {
        }
        
        virtual void draw()
        {
            if(visible)
            {
                glDisable(GL_COLOR_MATERIAL);
                glDisable(GL_LIGHTING);
                glEnable(GL_TEXTURE_2D);
                    if(pushing==0 && focus==true)
                        glBindTexture(GL_TEXTURE_2D,textures[0]);
                    else
                    if(pushing==1)
                        glBindTexture(GL_TEXTURE_2D,textures[1]);
                    else
                        glBindTexture(GL_TEXTURE_2D,textures[2]);
                    glBegin(GL_QUADS);								
        				glTexCoord2f(0,1);			
        				glVertex2i(x,y+height);		
        				glTexCoord2f(1,1);	
        				glVertex2i(x+width,y+height);
        				glTexCoord2f(1,0);			
        				glVertex2i(x+width,y);		
        				glTexCoord2f(0,0);			
        				glVertex2i(x,y);			
        			glEnd();			
                glDisable(GL_TEXTURE_2D);
                glEnable(GL_LIGHTING);
                glEnable(GL_COLOR_MATERIAL);
                
                text.draw();
            }
        }
};

