#include <GL/openglut.h>
#include <string>
using namespace std;


int LoadAndBuildFont(const char* file,int &textureN)									// Build Our Font Display List
{
	int base=glGenLists(256);									// Creating 256 Display Lists
	textureN=SOIL_load_OGL_texture
	(
		file,
		SOIL_LOAD_AUTO,
		SOIL_CREATE_NEW_ID,
		//SOIL_FLAG_POWER_OF_TWO | SOIL_FLAG_MIPMAPS | SOIL_FLAG_DDS_LOAD_DIRECT
		SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_LOAD_RGBA | SOIL_FLAG_COMPRESS_TO_DXT
	);
	glBindTexture(GL_TEXTURE_2D,textureN);
	for (int loop1=0; loop1<256; loop1++)						// Loop Through All 256 Lists
	{
		float cx=float(loop1%16)/16.0f;						// X Position Of Current Character
		float cy=float(loop1/16)/16.0f;						// Y Position Of Current Character

		glNewList(base+loop1,GL_COMPILE);					// Start Building A List
			glBegin(GL_QUADS);								// Use A Quad For Each Character
				glTexCoord2f(cx,1.0f-cy-0.0625f);			// Texture Coord (Bottom Left)
				glVertex2d(0,16);							// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f,1.0f-cy-0.0625f);	// Texture Coord (Bottom Right)
				glVertex2i(16,16);							// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f,1.0f-cy);			// Texture Coord (Top Right)
				glVertex2i(16,0);							// Vertex Coord (Top Right)
				glTexCoord2f(cx,1.0f-cy);					// Texture Coord (Top Left)
				glVertex2i(0,0);							// Vertex Coord (Top Left)
			glEnd();										// Done Building Our Quad (Character)
			glTranslated(15,0,0);							// Move To The Right Of The Character
		glEndList();										// Done Building The Display List
	}														// Loop Until All 256 Are Built
	return base;
}

class Font
{
    public:
        int base,texture;
        int xsize[256];
        int size;
        float scale;
        float r,g,b;
        Font(){}
        Font(const char* file)
        {
            base=LoadAndBuildFont(file,texture);
            scale=1.0;
            size=16;
            r=g=b=1;
            for(int i=0;i<256;i++)
            {
                xsize[i]=16;
            }
        }
        void setSize(int n)
        {
            scale=(float)n/(float)size;
            size=n;
        }
        void setColor(float R,float G,float B)
        {
            r=R;
            g=G;
            b=B;
        }
        float getLength(string str)
        {
            float length=0;
            for(int i=0;i<str.size();i++)
            {
                length+=((float)xsize[str[i]]*scale);
            }
            return length;
        }
        float getLength2(string str)
        {
            float length=0;
            for(int i=0;i<str.size();i++)
            {
                length+=((float)xsize[str[i]]*scale);
            }
            length+=((float)xsize[0]*scale);
            return length;
        }
};


void glPrint(GLint x, GLint y, Font f, const char *fmt, ...)	// Where The Printing Happens
{
	char		text[256];									// Holds Our String
	va_list		ap;											// Pointer To List Of Arguments

	if (fmt == NULL)										// If There's No Text
		return;												// Do Nothing

	va_start(ap, fmt);										// Parses The String For Variables
	    vsprintf(text, fmt, ap);							// And Converts Symbols To Actual Numbers
	va_end(ap);												// Results Are Stored In Text

	
	glEnable(GL_TEXTURE_2D);								// Enable Texture Mapping
	glBindTexture(GL_TEXTURE_2D,f.texture);
	glLoadIdentity();										// Reset The Modelview Matrix
	glTranslated(x,y,0);									// Position The Text (0,0 - Bottom Left)
	glListBase(f.base-32+(128*0));							// Choose The Font Set (0 or 1)

    glScalef(f.scale,f.scale,f.scale);
    glColor3f(f.r,f.g,f.b);
	glCallLists(strlen(text),GL_UNSIGNED_BYTE, text);		// Write The Text To The Screen
	glScalef(1,1,1);
	glDisable(GL_TEXTURE_2D);								// Disable Texture Mapping
}


static void shapesPrintf (int row, int col, const char *fmt, ...)
{
    static char buf[256];
    int viewport[4];
    void *font = GLUT_BITMAP_9_BY_15;
    va_list args;

    va_start(args, fmt);
    (void) vsprintf (buf, fmt, args);
    va_end(args);

    glGetIntegerv(GL_VIEWPORT,viewport);

    glPushMatrix();
    glLoadIdentity();

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();

        glOrtho(0,viewport[2],0,viewport[3],-1,1);

        glRasterPos2i(
              glutBitmapWidth(font, ' ') * col,
            - glutBitmapHeight(font) * (row+2) + viewport[3]
        );
        glutBitmapString (font, (unsigned char *) buf);

    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
}
